@@ -1,10 +1,10 @@
 #! /usr/bin/env python
-# encoding: ISO8859-1
+# encoding: utf-8
 # Thomas Nagy, 2014-2015
 
 """
 A simple file for verifying signatures in signed waf files
-This script is meant for Python >= 2.6 and the encoding is bytes - ISO8859-1
+This script is meant for Python >= 2.6 and the encoding is bytes - latin-1
 
 Distributing detached signatures is boring
 """
@@ -33,14 +33,14 @@ if __name__ == '__main__':
 	try:
 		txt = f.read()
 
-		lastline = txt.decode('ISO8859-1').splitlines()[-1] # just the last line
+		lastline = txt.decode('latin-1').splitlines()[-1] # just the last line
 		if not lastline.startswith('#-----BEGIN PGP SIGNATURE-----'):
 			print("ERROR: there is no signature to verify in %r :-/" % infile)
 			sys.exit(1)
 
 		sigtext = lastline.replace('\\n', '\n') # convert newlines
 		sigtext = sigtext[1:] # omit the '# character'
-		sigtext = sigtext.encode('ISO8859-1') # python3
+		sigtext = sigtext.encode('latin-1') # python3
 
 		f2.write(sigtext)
 		f1.write(txt[:-len(lastline) - 1]) # one newline character was eaten from splitlines()
