@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # encoding: utf-8
-# Thomas Nagy, 2006-2016 (ita)
+# Thomas Nagy, 2006-2018 (ita)
 
 """
 TeX/LaTeX/PDFLaTeX/XeLaTeX support
@@ -48,20 +48,23 @@ def bibunitscan(self):
 	node = self.inputs[0]
 
 	nodes = []
-	if not node: return nodes
+	if not node:
+		return nodes
 
 	code = node.read()
 	for match in re_bibunit.finditer(code):
 		path = match.group('file')
 		if path:
+			found = None
 			for k in ('', '.bib'):
 				# add another loop for the tex include paths?
 				Logs.debug('tex: trying %s%s', path, k)
 				fi = node.parent.find_resource(path + k)
 				if fi:
+					found = True
 					nodes.append(fi)
-					# no break, people are crazy
-			else:
+					# no break
+			if not found:
 				Logs.debug('tex: could not find %s', path)
 
 	Logs.debug('tex: found the following bibunit files: %s', nodes)
@@ -87,6 +90,7 @@ class tex(Task.Task):
 	Compiles a tex/latex file.
 
 	.. inheritance-diagram:: waflib.Tools.tex.latex waflib.Tools.tex.xelatex waflib.Tools.tex.pdflatex
+	   :top-classes: waflib.Tools.tex.tex
 	"""
 
 	bibtex_fun, _ = Task.compile_fun('${BIBTEX} ${BIBTEXFLAGS} ${SRCFILE}', shell=False)
@@ -156,14 +160,14 @@ class tex(Task.Task):
 		nodes = []
 		names = []
 		seen = []
-		if not node: return (nodes, names)
+		if not node:
+			return (nodes, names)
 
 		def parse_node(node):
 			if node in seen:
 				return
 			seen.append(node)
 			code = node.read()
-			global re_tex
 			for match in re_tex.finditer(code):
 
 				multibib = match.group('type')
@@ -351,9 +355,9 @@ class tex(Task.Task):
 		env = self.env
 
 		if not env.PROMPT_LATEX:
-			env.append_value('LATEXFLAGS', '-interaction=batchmode')
-			env.append_value('PDFLATEXFLAGS', '-interaction=batchmode')
-			env.append_value('XELATEXFLAGS', '-interaction=batchmode')
+			env.append_value('LATEXFLAGS', '-interaction=nonstopmode')
+			env.append_value('PDFLATEXFLAGS', '-interaction=nonstopmode')
+			env.append_value('XELATEXFLAGS', '-interaction=nonstopmode')
 
 		# important, set the cwd for everybody
 		self.cwd = self.inputs[0].parent.get_bld()
@@ -454,7 +458,7 @@ def apply_tex(self):
 
 	outs = Utils.to_list(getattr(self, 'outs', []))
 
-	# prompt for incomplete files (else the batchmode is used)
+	# prompt for incomplete files (else the nonstopmode is used)
 	try:
 		self.generator.bld.conf
 	except AttributeError:
